@@ -39,29 +39,6 @@
 
 static QofLogModule log_module = PQ_MOD_PILOT;
 
-#define DATEBOOK_VERSION datebook_v1
-#define DATEBOOK_EVENT        "untimed_event"
-#define DATEBOOK_BEGIN        "start_time"
-#define DATEBOOK_END          "end_time"
-#define DATEBOOK_ALARM        "use_alarm"
-#define DATEBOOK_ADVANCE      "alarm_advance"
-#define DATEBOOK_ADV_UNIT     "advance_unit"
-#define DATEBOOK_REPEAT_TYPE  "repeat_type"
-/** \brief Does the repeat have an end?
-
-If set, any value in repeat-end is ignored.
-*/
-#define DATEBOOK_REPEAT_FOREVER    "repeat_forever"
-#define DATEBOOK_REPEAT_END        "repeat_end"
-#define DATEBOOK_REPEAT_FREQUENCY  "repeat_frequency"
-#define DATEBOOK_REPEAT_DAY        "repeat_day"
-#define DATEBOOK_REPEAT_WEEK_START "repeat_week_start"
-#define DATEBOOK_EXCEPTIONS        "exception_count"
-#define DATEBOOK_EXCEPTION         "exception_list"
-#define DATEBOOK_NOTE              "note"
-#define DATEBOOK_CATEGORY          "category"
-#define DATEBOOK_KVP_PATH          "datebook/exceptions"
-
 /** Wraps QOF around the libpisock object. */
 typedef struct
 {
@@ -157,6 +134,14 @@ datebook_create (QofBook * book)
 	return obj;
 }
 
+Appointment_t * datebook_get_pilot (QofInstance * inst)
+{
+	QofDateBook * d;
+
+	d = (QofDateBook *)inst;
+	return &d->wrap;
+}
+
 static gboolean
 datebook_getEvent (QofDateBook * d)
 {
@@ -638,7 +623,7 @@ datebook_setDescription (QofDateBook * d, gchar * h)
 
 	g_return_if_fail (d);
 	qd = &d->wrap;
-	qd->description = g_strdup (qof_main_make_utf8 (h));
+	qd->description = g_strdup (qof_util_make_utf8 (h));
 }
 
 static void
@@ -650,289 +635,45 @@ datebook_setNote (QofDateBook * d, gchar * h)
 	qd = &d->wrap;
 	if (!h)
 		return;
-	qd->note = g_strdup (qof_main_make_utf8 (h));
+	qd->note = g_strdup (qof_util_make_utf8 (h));
 }
 
 static void
 datebook_setCategory (QofDateBook * d, gchar * n)
 {
 	g_return_if_fail (d != NULL);
-	d->category = g_strdup (qof_main_make_utf8 (n));
+	d->category = g_strdup (qof_util_make_utf8 (n));
 }
 
-/** unpack the Palm datebook record into the entity. 
-
-\todo Problems with queries not finding repeat events.
-*/
-
-static gint
-datebook_unpack (QofEntity * ent, gpointer user_data)
+void
+datebook_repeater_clone (QofEntity * ent, QofTime * qt_end, QofTime * qt_increment)
 {
-	pi_buffer_t *pi_buf;
-	Appointment_t *qa;
-	QofDateBook *obj, *clone;
-	QofInstance *inst;
-	QofBook *book;
-	KvpFrame *inst_frame;
-	QofTime *qt, *qt_increment, *qt_end, *qt_repeat;
-	PQContext *context;
-	gint size, i, day_interval, month_interval;
+	QofDateBook * clone;
+	QofInstance * inst;
+	QofBook * book;
+	QofDateBook * obj;
 
-	context = (PQContext *) user_data;
-	day_interval = 0;
-	month_interval = 0;
-	g_return_val_if_fail (context != NULL, -1);
-	g_return_val_if_fail (ent != NULL, -1);
-	obj = (QofDateBook *) ent;
-	inst = (QofInstance *) ent;
-	inst_frame = qof_instance_get_slots (inst);
-	qa = &obj->wrap;
-	pi_buf = (pi_buffer_t *) context->pi_buf;
-	size = 0;
-	size = unpack_Appointment (qa, pi_buf, DATEBOOK_VERSION);	// 0.12
-	datebook_setCategory (obj, context->names[context->ent_category]);
-	/* Use <= and handle zero if < omits last entry (&623) */
-	for (i = 0; i < qa->exceptions; i++)
-	{
-		gchar *extend;
-
-		extend = NULL;
-		DEBUG (" exceptions=%d", qa->exceptions);
-		qt = qof_time_from_tm (&qa->exception[i], 0);
-		extend = g_strdup_printf ("%s/%d", DATEBOOK_KVP_PATH, i + 1);
-		kvp_frame_set_time (inst_frame, extend, qt);
-		inst->kvp_data = inst_frame;
-		g_free (extend);
-	}
-	if (qa->repeatType == repeatNone)
-		qa->repeatEnd = qa->end;
-	/* set a marker for the interval. Do the iteration once, outside the switch. */
-	switch (qa->repeatType)
-	{
-	case repeatNone:
-		{
-			day_interval = 0;
-			month_interval = 0;
-			break;
-		}
-	case repeatDaily:
-		{
-			day_interval = 1;
-			month_interval = 0;
-			break;
-		}
-	case repeatWeekly:
-		{
-			day_interval = 7;
-			month_interval = 0;
-			break;
-		}
-	case repeatMonthlyByDay:
-	case repeatMonthlyByDate:
-		{
-			day_interval = 0;
-			month_interval = 1;
-			break;
-		}
-	case repeatYearly:
-		{
-			day_interval = 0;
-			month_interval = 12;
-			break;
-		}
-	default:
-		{
-			PERR (" unsupported repeatType=%d", qa->repeatType);
-		}
-	}
-	if (day_interval == 0 && month_interval == 0)
-		return size;
-	/* Now create a repeater in the SAME book. */
-	qt = datebook_getBegin (obj);
-	qt_end = datebook_getEnd (obj);
-	if (qa->repeatForever == 0)
-	{
-		qt_repeat = datebook_getRepeatEnd (obj);
-	}
-	else
-	{
-		QofDate *qd;
-		/*  if qa->repeatForever == 1 (true), calculate year and a day from
-		   start_time. qof_date_add_months(12)
-		   qof_date_add_days(1). Store for use as repeatEnd
-		 */
-		DEBUG (" qa->repeatForever == 1");
-		qt_repeat = qt;
-		qd = qof_date_from_qtime (qt_repeat);
-		qof_date_addmonths (qd, 12, FALSE);
-		qof_date_adddays (qd, 1);
-		qof_date_free (qd);
-	}
-	qt_increment = qt;
-	/*  qa->exception is an array of struct tm* qa->exceptions long. */
-	/* while datebook_getBegin is less (earlier) than repeat_end */
-	while (qof_time_cmp (qt_increment, qt_repeat) < 0)
-	{
-		gboolean skip;
-
-		skip = FALSE;
-		if (day_interval)
-		{
-			QofDate *qd;
-			qd = qof_date_from_qtime (qt_increment);
-			qof_date_adddays (qd, day_interval);
-			qt_increment = qof_date_to_qtime (qd);
-			qof_date_free (qd);
-			qd = qof_date_from_qtime (qt_end);
-			qof_date_adddays (qd, day_interval);
-			qt_end = qof_date_to_qtime (qd);
-			qof_date_free (qd);
-		}
-		if (month_interval)
-		{
-			QofDate *qd;
-			qd = qof_date_from_qtime (qt_increment);
-			qof_date_addmonths (qd, month_interval, FALSE);
-			qt_increment = qof_date_to_qtime (qd);
-			qof_date_free (qd);
-			qd = qof_date_from_qtime (qt_end);
-			qof_date_addmonths (qd, month_interval, FALSE);
-			qt_end = qof_date_to_qtime (qd);
-			qof_date_free (qd);
-		}
-		for (i = 0; i < qa->exceptions; i++)
-		{
-			QofDate *qd;
-
-			qd = qof_date_from_qtime(qt_increment);
-			if ((qd->qd_year == qa->exception[i].tm_year) &&
-				(qd->qd_mday == qa->exception[i].tm_mday) &&
-				(qd->qd_mon == qa->exception[i].tm_mon))
-			{
-				/* exclude */
-				skip = TRUE;
-			}
-		}
-		/* create the repeater */
-		if (!skip)
-		{
-			book = inst->book;
-			clone = datebook_create (book);
-			clone->repeater = TRUE;
-			/* clone the main parts of the event */
-			datebook_setEvent (clone, datebook_getEvent (obj));
-			datebook_setAlarm (clone, datebook_getAlarm (obj));
-			datebook_setAdvance (clone, datebook_getAdvance (obj));
-			datebook_setAdvanceUnit (clone, datebook_getAdvanceUnit (obj));
-			datebook_setDescription (clone, datebook_getDescription (obj));
-			datebook_setNote (clone, datebook_getNote (obj));
-			datebook_setCategory (clone, datebook_getCategory (obj));
-			/* the clone does not repeat */
-			datebook_setExceptions (clone, 0);
-			datebook_setRepeatEnd (clone, qt_end);
-			datebook_setRepeatForever (clone, FALSE);
-			datebook_setRepeatType (clone,
-				repeatTypesasString (repeatNone));
-			datebook_setBegin (clone, qt_increment);
-			datebook_setEnd (clone, qt_end);
-		}
-	}
-	return size;
-}
-
-/** \brief pack the entity into a Palm datebook record */
-static gint
-datebook_pack (QofEntity * ent, gpointer user_data)
-{
-	PQContext *context;
-	Appointment_t *qa;
-	QofDateBook *obj;
-	QofTime *qt;
-	KvpFrame *frame;
-	gint size, i;
-	gchar *path;
-
-	size = 0;
-	i = 1;
-	context = (PQContext *) user_data;
-	g_return_val_if_fail ((context || ent), -1);
-	ENTER (" ");
-	obj = (QofDateBook *) ent;
-	if (obj->repeater == TRUE)
-		return 0;
-	qa = &obj->wrap;
-	size = pack_Appointment (qa, context->pi_buf, DATEBOOK_VERSION);	// 0.12
-	/* pack slots into exceptions */
-	frame = qof_instance_get_slots ((QofInstance *) ent);
-	if (frame)
-	{
-		path = g_strdup_printf ("%s/%d", DATEBOOK_KVP_PATH, i);
-		while (kvp_frame_get_value (frame, path))
-		{
-			qt = kvp_frame_get_time (frame, path);
-			if (qt)
-			{
-				QofDate *qd;
-				gboolean result;
-				qd = qof_date_from_qtime (qt);
-				result = qof_date_to_struct_tm (qd,
-					&qa->exception[i], 0);
-				if(!result)
-					PERR (" failed to set exception time: "
-					"out of range");
-			}
-			g_free (path);
-			i++;
-			path = g_strdup_printf ("%s/%d", DATEBOOK_KVP_PATH, i);
-		}
-		g_free (path);
-	}
-	LEAVE (" ");
-	return size;
-}
-
-/** \brief unpack the application information.
-
-The AppInfo contains the category list (each record only holds
-the index value of this list as the category value) and other
-preferences.
-*/
-static gint
-datebook_appinfo_unpack (QofEntity * ent, gpointer user_data)
-{
-	AppointmentAppInfo_t app_db;
-	PQContext *context;
-	gint name_count;
-
-	context = (PQContext *) user_data;
-	g_return_val_if_fail (context != NULL, -1);
-	ENTER (" ");
-	unpack_AppointmentAppInfo (&app_db, context->app_buf->data,
-		PQ_DEF_BUFSZ);
-	for (name_count = 0; name_count < 16; name_count++)
-	{
-		g_sprintf (context->names[name_count], "%s",
-			app_db.category.name[name_count]);
-	}
-	context->pi_cat = &app_db.category;
-	LEAVE (" ");
-	return 0;
-}
-
-/** free all memory related to the datebook record. */
-static gint
-qof_datebook_free (QofEntity * ent, gpointer user_data)
-{
-	Appointment_t *qd;
-	QofDateBook *obj;
-
-	g_return_val_if_fail (ent != NULL, -1);
-	ENTER (" ");
-	obj = (QofDateBook *) ent;
-	qd = &obj->wrap;
-	free_Appointment (qd);
-	LEAVE (" ");
-	return 0;
+	obj = (QofDateBook*)ent;
+	inst = (QofInstance*)obj;
+	book = inst->book;
+	clone = datebook_create (book);
+	clone->repeater = TRUE;
+	/* clone the main parts of the event */
+	datebook_setEvent (clone, datebook_getEvent (obj));
+	datebook_setAlarm (clone, datebook_getAlarm (obj));
+	datebook_setAdvance (clone, datebook_getAdvance (obj));
+	datebook_setAdvanceUnit (clone, datebook_getAdvanceUnit (obj));
+	datebook_setDescription (clone, datebook_getDescription (obj));
+	datebook_setNote (clone, datebook_getNote (obj));
+	datebook_setCategory (clone, datebook_getCategory (obj));
+	/* the clone does not repeat */
+	datebook_setExceptions (clone, 0);
+	datebook_setRepeatEnd (clone, qt_end);
+	datebook_setRepeatForever (clone, FALSE);
+	datebook_setRepeatType (clone,
+		repeatTypesasString (repeatNone));
+	datebook_setBegin (clone, qt_increment);
+	datebook_setEnd (clone, qt_end);
 }
 
 static const gchar *
@@ -955,15 +696,6 @@ static QofObject datebook_object_def = {
   version_cmp:(gint (*)(gpointer, gpointer)) qof_instance_version_cmp,
 };
 
-static PQPack datebook_pack_def = {
-  e_type:PILOT_LINK_QOF_DATEBOOK,
-  pack_func:datebook_pack,
-  unpack_func:datebook_unpack,
-  free_pack_func:qof_datebook_free,
-  palm_db_name:"DatebookDB",
-  app_info_unpack:datebook_appinfo_unpack,
-};
-
 gboolean
 DateBookRegister (void)
 {
@@ -1029,8 +761,6 @@ DateBookRegister (void)
 
 	qof_class_register (PILOT_LINK_QOF_DATEBOOK, NULL, params);
 
-	pilot_qof_pack_register (&datebook_pack_def);
-
 	return qof_object_register (&datebook_object_def);
 }
 
